@@ -28,7 +28,7 @@ When fully operational, it will use an algorithm along the lines of the followin
     * How do we merge together the two static roots?
 * If static files go through Jifty::Handler
     * We need a flag to allow them to go through the dispatcher, too
-    * return "True" (304) for if-modified-since unless develmode.
+    * return "True" (304) for if-modified-since
     * if the browser accepts gzipped data,
         see if we have a cached gzipped copy
             if so, send it
@@ -67,17 +67,24 @@ sub handle_request {
     my $self = shift;
     my $path = shift;
 
-    #if ( Jifty->handler->cgi->http('If-Modified-Since') and not Jifty->config->framework('DevelMode') ) { $self->respond_not_modified(); }
-    my $local_path = $self->file_path($path);
-    unless ($local_path) {
-        return undef;
+    my $local_path = $self->file_path($path) or return undef;
+
+    if ( my $since = Jifty->handler->cgi->http('If-Modified-Since') ) {
+        my @file_info = stat($local_path);
+
+        # IE appends "; length=N" to If-Modified-Since headers and we need
+        # to get rid of it so str2time doesn't choke below
+        $since =~ s/;.+$//;
+
+        return $self->send_not_modified
+            unless $file_info[9] > HTTP::Date::str2time($since);
     }
     my $mime_type = $self->mime_type($local_path);
-    
+
     if ( $self->client_accepts_gzipped_content and $mime_type =~ m!^(text/|application/x-javascript)! ) {
         return $self->send_file($local_path, $mime_type, 'gzip');
     } else {
-        return  $self->send_file($local_path, $mime_type, 'uncompressed');
+        return $self->send_file($local_path, $mime_type, 'uncompressed');
     }
 
 }
@@ -228,7 +235,9 @@ Sends a "304 Not modified" response to the browser, telling it to use a cached c
 
 sub send_not_modified {
     my $self = shift;
-    Jifty->handler->apache->header_out( Status => 304 );
+    my $apache = Jifty->handler->apache;
+    $apache->header_out( Status => 304 );
+    $apache->send_http_header();
     return 1;
 
 }
