@@ -104,8 +104,8 @@ C<new>.  Subclasses should extend this list.
 
 =cut
 
-sub accessors { shift->SUPER::accessors(), qw(name label input_name type sticky sticky_value default_value action mandatory ajax_validates ajax_canonicalizes autocompleter preamble hints placeholder render_mode length _element_id); }
-__PACKAGE__->mk_accessors(qw(name _label _input_name type sticky sticky_value default_value _action mandatory ajax_validates ajax_canonicalizes autocompleter preamble hints placeholder render_mode length _element_id));
+sub accessors { shift->SUPER::accessors(), qw(name label input_name type sticky sticky_value default_value action mandatory ajax_validates ajax_canonicalizes autocompleter preamble hints placeholder focus render_mode length _element_id); }
+__PACKAGE__->mk_accessors(qw(name _label _input_name type sticky sticky_value default_value _action mandatory ajax_validates ajax_canonicalizes autocompleter preamble hints placeholder focus render_mode length _element_id));
 
 =head2 name [VALUE]
 
@@ -129,6 +129,10 @@ Gets or sets the type of the HTML <input> field -- that is, 'text' or
 
 Sets this form field's "submit" key binding to VALUE. 
 
+=head2 key_binding_label VALUE
+
+Sets this form field's key binding label to VALUE.  If none is specified
+the normal label is used.
 
 =head2 default_value [VALUE]
 
@@ -143,6 +147,10 @@ Gets or sets the value for the form field that was submitted in the last action.
 A boolean indicating that the argument B<must> be present when the
 user submits the form.
 
+=head2 focus [VALUE]
+
+If true, put focus on this form field when the page loads.
+
 =head2 ajax_validates [VALUE]
 
 A boolean value indicating if user input into an HTML form field for
@@ -161,6 +169,10 @@ form, instead of waiting until submit.  Arguments will B<always> be
 canonicalized before the action is run, whether or not they also
 C<ajax_canonicalize>
 
+=head2 preamble [VALUE]
+
+Gets or sets the preamble located in front of the field.
+
 =head2 id 
 
 For the purposes of L<Jifty::Web::Form::Element>, the unique id of
@@ -237,6 +249,24 @@ sub label {
 
 }
 
+=head2 hints [VALUE]
+
+Hints for the user to explain this field
+
+=cut
+
+sub hints {
+    my $self = shift;
+    return $self->_hints_accessor unless @_;
+
+    my $hint = shift;
+    # people sometimes say hints are "foo" rather than hints is "foo"
+    if (ref $hint eq 'ARRAY') {
+        $hint = shift @$hint;
+    }
+    return $self->_hints_accessor($hint);
+}
+
 
 =head2 element_id 
 
@@ -312,12 +342,12 @@ sub render {
     $self->render_label();
     if ($self->render_mode eq 'update') { 
         $self->render_widget();
-        $self->render_autocomplete();
-        $self->render_placeholder();
-        $self->render_key_binding();
+        $self->render_autocomplete_div();
+        $self->render_inline_javascript();
         $self->render_hints();
         $self->render_errors();
         $self->render_warnings();
+        $self->render_canonicalization_notes();
     } elsif ($self->render_mode eq 'read'){ 
         $self->render_value();
     }
@@ -325,6 +355,36 @@ sub render {
     return ('');
 }
 
+=head2 render_inline_javascript
+
+Render a <script> tag (if neceesary) containing any inline javascript
+that should follow this form field. This is used to add an
+autocompleter, placeholder, or keybinding to form fields where needed.
+
+=cut
+
+sub render_inline_javascript {
+    my $self = shift;
+
+    my $javascript;
+
+    $javascript = join(
+        "\n",
+        grep {$_} (
+            $self->autocomplete_javascript(),
+            $self->placeholder_javascript(),
+            $self->key_binding_javascript(),
+            $self->focus_javascript()
+        )
+    );
+    
+    if($javascript =~ /\S/) {
+        Jifty->web->out(qq{<script type="text/javascript"><!--
+    $javascript
+--></script>
+});
+    }
+}
 
 =head2 classes
 
@@ -379,7 +439,7 @@ Use this for sticking instructions right in front of a widget
 sub render_preamble {
     my $self = shift;
     Jifty->web->out(
-qq!<span class="preamble @{[$self->classes]}" >@{[_($self->preamble) || '' ]}</span>\n!
+qq!<span class="preamble @{[$self->classes]}">@{[_($self->preamble) || '' ]}</span>\n!
     );
 
     return '';
@@ -423,9 +483,9 @@ sub render_widget {
     $field .= qq! type="@{[ $self->type ]}"!;
     $field .= qq! name="@{[ $self->input_name ]}"! if ($self->input_name);
     $field .= qq! id="@{[ $self->element_id ]}"!;
-    $field .= qq! value="@{[HTML::Entities::encode_entities($self->current_value)]}"! if defined $self->current_value;
+    $field .= qq! value="@{[Jifty->web->escape($self->current_value)]}"! if defined $self->current_value;
     $field .= $self->_widget_class; 
-    $field .= qq! size="@{[ $self->length() ]}"! if ($self->length());
+    $field .= qq! size="@{[ $self->length() ]}" maxlength="@{[ $self->length() ]}"! if ($self->length());
     $field .= " " .$self->other_widget_properties;
     $field .= qq!  />\n!;
     Jifty->web->out($field);
@@ -462,6 +522,7 @@ sub _widget_class {
                     ( $self->ajax_validates     ? ' ajaxvalidation' : '' ),
                     ( $self->ajax_canonicalizes ? ' ajaxcanonicalization' : '' ),
                     ( $self->autocompleter      ? ' ajaxautocompletes' : '' ),
+                    ( $self->focus              ? ' focus' : ''),
                     @_ );
 
     return qq! class="!. join(' ',@classes).  qq!"!
@@ -480,7 +541,7 @@ sub render_value {
     my $field = '<span';
     $field .= qq! class="@{[ $self->classes ]}"> !;
     # XXX: force stringify the value because maketext is buggy with overloaded objects.
-    $field .= HTML::Entities::encode_entities(_("@{[$self->current_value]}")) if defined $self->current_value;
+    $field .= Jifty->web->escape(_("@{[$self->current_value]}")) if defined $self->current_value;
     $field .= qq!</span>\n!;
     Jifty->web->out($field);
     return '';
@@ -488,50 +549,89 @@ sub render_value {
 
 
 
-=head2 render_autocomplete
+=head2 render_autocomplete_div
 
-Renders an empty div that /__jifty/autocomplete.xml can fill in. Also renders the tiny snippet
-of javascript to make that call if necessary.
-Returns an empty string.
+Renders an empty div that /__jifty/autocomplete.xml can fill
+in. Returns an empty string.
 
 =cut
 
-sub render_autocomplete { 
+sub render_autocomplete_div { 
     my $self = shift;
     return unless($self->autocompleter);
     Jifty->web->out(
-qq!<div class="autocomplete" id="@{[$self->element_id]}-autocomplete" style="display: none;"></div>\n
-        <script type="text/javascript">
-          new Jifty.Autocompleter('@{[$self->element_id]}','@{[$self->element_id]}-autocomplete')
-        </script>
-  !
-    );
+qq!<div class="autocomplete" id="@{[$self->element_id]}-autocomplete" style="display: none;"></div>!);
 
     return '';
-
 }
 
-=head2 render_placeholder
+=head2 render_autocomplete
 
-Renders the javascript necessary to insert a placeholder into this
-form field (greyed-out text that is written in using javascript, and
-vanishes when the user focuses the field). Returns an empty string.
+Renders the div tag and javascript necessary to do autocompletion for
+this form field. Deprecated internally in favor of
+L</render_autocomplete_div> and L</autocomplete_javascript>, but kept
+for backwards compatability since there exists external code that uses
+it.
 
 =cut
 
-sub render_placeholder {
+sub render_autocomplete {
+    my $self = shift;
+    return unless $self->autocompleter;
+    $self->render_autocomplete_div;
+    Jifty->web->out(qq{<script type="text/javascript"><!--
+    @{[$self->autocomplete_javascript]}
+--></script>});
+    return '';
+}
+
+
+
+=head2 autocomplete_javascript
+
+Returns renders the tiny snippet of javascript to make an autocomplete
+call, if necessary.
+
+=cut
+
+sub autocomplete_javascript {
+    my $self = shift;
+    return unless($self->autocompleter);
+    return qq{new Jifty.Autocompleter('@{[$self->element_id]}','@{[$self->element_id]}-autocomplete')};
+}
+
+=head2 placeholder_javascript
+
+Returns the javascript necessary to insert a placeholder into this
+form field (greyed-out text that is written in using javascript, and
+vanishes when the user focuses the field). 
+
+=cut
+
+sub placeholder_javascript {
     my $self = shift;
     return unless $self->placeholder;
     my $placeholder = $self->placeholder;
     $placeholder =~ s{(['\\])}{\\$1}g;
     $placeholder =~ s{\n}{\\n}g;
     $placeholder =~ s{\r}{\\r}g;
-    Jifty->web->out(
-qq!<script type="text/javascript">
-     new Jifty.Placeholder('@{[$self->element_id]}', '$placeholder');
-   </script>
-!
-        );
+    return qq{new Jifty.Placeholder('@{[$self->element_id]}', '$placeholder');};
+}
+
+=head2 focus_javascript
+
+Returns the javascript necessary to focus this form field on page
+load, if necessary.
+
+=cut
+
+sub focus_javascript {
+    my $self = shift;
+    return undef;
+    if($self->focus) {
+        return qq{document.getElementById("@{[$self->element_id]}").focus()};
+        return qq{DOM.Events.addListener( window, "load", function(){document.getElementById("@{[$self->element_id]}").focus()})};
+    }
 }
 
 =head2 render_hints
@@ -588,4 +688,22 @@ qq!<span class="warning @{[$self->classes]}" id="@{[$self->action->warning_div_i
     return '';
 }
 
+=head2 render_canonicalization_notes
+
+Outputs a <div> with any canonicalization notes for this action, even if there are
+none -- AJAX could fill it in.
+
+=cut
+
+sub render_canonicalization_notes {
+    my $self = shift;
+
+    return unless $self->action;
+
+    Jifty->web->out(
+qq!<span class="canonicalization_note @{[$self->classes]}" id="@{[$self->action->canonicalization_note_div_id($self->name)]}">@{[$self->action->result->field_canonicalization_note( $self->name ) || '']}</span>\n!
+    );
+    return '';
+}
+
 1;
